/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define ArmeDataFileRef_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

class ArmeArchiveHandler;
class TArmeDoc;
/*
I have added a support for data file references (audio or video).  It means that each clips keeps a reference on every file it uses such that if any file is not referenced any more, it can be removed from the project directory.

For that purpose, I have created the ArmeDataFileRef class and subclass ArmeVideoDataFileRef and ArmeAudioDataFileRef.
This class simply contains a fileIndex (given by the document) and a reference counter.
You can ask the doc for a GetNewAudioDataFileRef() and then get the filename with ref->GetFilePath();
*/
class ArmeDataFileRef : public ArmeArchivable
{
	UInt32					fNbReferences;
	virtual const char*		GetFileNameBase() = 0;

	char*				fHighResSourceFile;				//the path of the high ressource data 
	ArmeDataFileRef*	fHighResSourceDataFileRef;		//a pointer to the high res source data  (only one of these 2 members is used)

	bool 				fHasBeenCreatedByMontage;
	char*				fSourceFilePath;						//complete path of the file referenced
	bool				fIsReferenceToExternalFile;

	void				Init();
public :
	bool			WriteToArchive(ArmeArchiveHandler* archiver) const;
	
	UInt32			fFileIndex;
	TArmeDoc*		fDoc;

	void			AddRef();
	void			Release();
	UInt32			GetRefCount() const;
	void			GetFilePath(char* destString);
	void			GetFilePathInSelfContainedProject(char* destString);
	void			SetHighResSourceFile(const char* source);
	void			SetHighResSourceFile(ArmeDataFileRef*	dataFileRef);
	const char*		GetHighResSourceFile();

	const char* 	GetSourceFilePath();
	void			SetSourceFilePath(const char* filePath);
	void			SetReferenceToExternalFile();
	void			UpdateForSelfContainedProject();
	bool			IsReferenceToExternalFile() const;

	virtual bool	IsVideoRef() const;

	ArmeDataFileRef(UInt32 fileIndex, TArmeDoc* doc, const char* sourceFile = NULL);
	ArmeDataFileRef(ArmeArchiveHandler* unarchiver);
	virtual ~ArmeDataFileRef();

	void 			PreDestructor(); //$$$BENJA hack for the vtable problem.
};

class ArmeVideoDataFileRef : public ArmeDataFileRef
{
public :
	virtual const char*		GetFileNameBase();
	static void				GetFilePathForIndex(const TArmeDoc* doc, char* destString, int index);
	
	ArmeVideoDataFileRef(UInt32 fileIndex, TArmeDoc* doc, const char* sourceFile = NULL);
	ArmeVideoDataFileRef(ArmeArchiveHandler* unarchiver);

	virtual bool	IsVideoRef() const;
};

/*
class ArmeAudioDataFileRef : public ArmeDataFileRef
{
public :
	virtual const char*		GetFileNameBase();
	
	ArmeAudioDataFileRef(UInt32 fileIndex, TArmeDoc* doc);
	ArmeAudioDataFileRef(ArmeArchiveHandler* unarchiver);
};
*/

class ArmeVideoTakeDataFileRef : public ArmeVideoDataFileRef
{
public :
	virtual const char*		GetFileNameBase();
	void GetFilePath(char* destString);
	ArmeVideoTakeDataFileRef(UInt32 fileIndex, TArmeDoc* doc);
	ArmeVideoTakeDataFileRef(ArmeArchiveHandler* unarchiver);
};

/*!
	@class ZzRABI_ArmeDataFileRef_Release
	
	This class calls, in its dtor, "fileRef->Release()" on an ArmeFileRef* ptr supplied in its ctor

*/

class ZzRABI_ArmeDataFileRef_Release
{
public :
	ZzRABI_ArmeDataFileRef_Release(ArmeDataFileRef* fileRef) { fFileRef = fileRef; }
	~ZzRABI_ArmeDataFileRef_Release() { fFileRef->Release(); }
private :
	ArmeDataFileRef* fFileRef;
};


